from . import Project
from . import ComObjectWrapper
from . import w32com
from typing import Union

class FDSolver(ComObjectWrapper):
    OBC_HEX_DEFAULT = 'Default'
    OBC_HEX_PML = 'PML'
    OBC_HEX_FREESPACE_SIBC = 'FreespaceSIBC'

    OBC_TET_DEFAULT = 'Default'
    OBC_TET_PML = 'PML'
    OBC_TET_SIBC = 'SIBC'

    RESET_SAMPLE_INTERVALS_ALL = 'all'
    RESET_SAMPLE_INTERVALS_ADAPTATION = 'adaptation'
    RESET_SAMPLE_INTERVALS_SINGLE = 'single'
    RESET_SAMPLE_INTERVALS_INACTIVE = 'inactive'
    RESET_SAMPLE_INTERVALS_INFINITE = 'infinite'

    SAMPLE_INTERVAL_AUTOMATIC = 'Automatic'
    SAMPLE_INTERVAL_SINGLE = 'Single'
    SAMPLE_INTERVAL_EQUIDISTANT = 'Equidistant'
    SAMPLE_INTERVAL_LOGARITHMIC = 'Logarithmic'

    FREQ_DIST_ADAPT_MODE_LOCAL = 'Local'
    FREQ_DIST_ADAPT_MODE_AS_A_WHOLE = 'As_A_Whole'
    FREQ_DIST_ADAPT_MODE_DISTRIBUTED = 'Distributed'

    TYPE_AUTO = 'Auto'
    TYPE_ITERATIVE = 'Iterative'
    TYPE_DIRECT = 'Direct'

    METHOD_MESH_HEXAHEDRAL = 'Hexahedral'
    METHOD_MESH_TETRAHEDRAL = 'Tetrahedral'
    METHOD_MESH_SURFACE = 'Surface'

    METHOD_SWEEP_GENERAL_PURPOSE = 'General purpose'
    METHOD_SWEEP_FAST_REDUCED_ORDER_MODEL = 'Fast reduced order model'
    METHOD_SWEEP_DISCRETE_SAMPLES_ONLY = 'Discrete samples only'

    STIMULATION_ALL = 'All'
    STIMULATION_ALL_PLUS_FLOQUET = 'All+Floquet'
    STIMULATION_PLANE_WAVE = 'Plane Wave'
    STIMULATION_LIST = 'List'
    STIMULATION_CMA = 'CMA'

    RESULT_DATA_SAMPLING_AUTO = 'Automatic'
    RESULT_DATA_SAMPLING_FREQ_LIN = 'Frequency (linear)'
    RESULT_DATA_SAMPLING_FREQ_LOG = 'Frequency (logarithmic)'
    RESULT_DATA_SAMPLING_FREQ_LOG_LIN = 'Frequency (log-linear)'

    EXCITATION_PORT_ZMIN = 'zmin'
    EXCITATION_PORT_ZMAX = 'zmax'

    EXCITATION_MODE_1 = '1'
    EXCITATION_MODE_1_3_4 = '1;3;4'
    EXCITATION_MODE_TE00 = 'TE(0,0)'
    EXCITATION_MODE_TM00 = 'TM(0,0)'
    EXCITATION_MODE_LCP = 'LCP'
    EXCITATION_MODE_RCP = 'RCP'

    ORDER_TET_FIRST = 'First'
    ORDER_TET_SECOND = 'Second'
    ORDER_TET_THIRD = 'Third'

    ORDER_SRF_FIRST = 'First'
    ORDER_SRF_SECOND = 'Second'
    ORDER_SRF_THIRD = 'Third'

    NETWORK_COMPUTING_STRATEGY_RUN_REMOTE = 'RunRemote'
    NETWORK_COMPUTING_STRATEGY_SAMPLES = 'Samples'

    MLFFM_ACCURACY_VERY_LOW_MEM = 'VeryLowMem'
    MLFFM_ACCURACY_LOW_MEM = 'LowMem'
    MLFFM_ACCURACY_DEFAULT = 'Default'
    MLFFM_ACCURACY_HIGH_ACC = 'HighAcc'

    def __init__(self, project) -> None:
        self.project: Project = project
        self.com_object = project.com_object.FDSolver

    def invoke_method(self, name, *args, **kwargs):
        self.project.ensure_active()
        return super().invoke_method(name, *args, **kwargs)

    def reset(self):
        self.invoke_method('Reset')

    def start(self) -> bool:
        success = self.invoke_method('Start')
        return success != 0

    def set_accelerated_restart(self, flag: bool = True):
        self.invoke_method('AcceleratedRestart', flag)

    def set_accuracy_tet(self, value: float):
        self.invoke_method('AccuracyTet', value)

    def set_accuracy_srf(self, value: float):
        self.invoke_method('AccuracySrf', value)

    def set_accuracy_hex(self, value: float):
        self.invoke_method('AccuracyHex', value)

    def set_accuracy_rom(self, value: float):
        self.invoke_method('AccuracyROM', value)

    def set_store_all_results(self, flag: bool = True):
        self.invoke_method('StoreAllResults', flag)

    def set_store_solution_coefficients(self, flag: bool = True):
        self.invoke_method('StoreSolutionCoefficients', flag)

    def set_create_legacy_1d_signals(self, flag: bool = True):
        self.invoke_method('CreateLegacy1DSignals', flag)

    def set_obc_type_hex(self, obc_type: str):
        self.invoke_method('SetOpenBCTypeHex', obc_type)

    def set_obc_type_tet(self, obc_type: str):
        self.invoke_method('SetOpenBCTypeTet', obc_type)

    def set_add_monitor_samples(self, flag: bool = True):
        self.invoke_method('AddMonitorSamples', flag)

    def set_max_number_of_frequency_samples(self, number: int):
        self.invoke_method('FrequencySamples', number)

    def set_mesh_adaptation_hex(self, flag: bool = True):
        self.invoke_method('MeshAdaptionHex', flag)

    def set_mesh_adaption_tet(self, flag: bool = True):
        self.invoke_method('MeshAdaptionTet', flag)

    def reset_sample_intervals(self, key: str):
        self.invoke_method('ResetSampleIntervals', key)

    def set_use_helmholtz_equation(self, flag: bool = True):
        self.invoke_method('UseHelmholtzEquation', flag)

    def add_sample_interval(self, min: float, max: float, num_samples: int, key: str):
        self.invoke_method('AddSampleInterval', min, max, num_samples, key, False)

    def add_sample_interval_with_adaptation(
            self, min: float, max: float, num_samples: int, key: str):
        self.invoke_method('AddSampleInterval', min, max, num_samples, key, True)

    def add_inactive_sample_interval(self, min: float, max: float, num_samples: int, key: str):
        self.invoke_method('AddInactiveSampleInterval', min, max, num_samples, key, False)

    def add_inactive_sample_interval_with_adaptation(
            self, min: float, max: float, num_samples: int, key: str):
        self.invoke_method('AddInactiveSampleInterval', min, max, num_samples, key, True)

    def set_max_number_of_iterations(self, count: int):
        self.invoke_method('MaxIterations', count)

    def set_limit_iterations(self, flag: bool = True):
        self.invoke_method('LimitIterations', flag)

    def set_modes_only(self, flag: bool = True):
        self.invoke_method('ModesOnly', flag)

    def set_shield_all_ports(self, flag: bool = True):
        self.invoke_method('SetShieldAllPorts', flag)

    def set_port_mesh_matches_3d_mesh_tet(self, flag: bool = True):
        self.invoke_method('SetPortMeshMatches3DMeshTet', flag)

    def set_allow_rom_port_mode_solver(self, flag: bool = True):
        self.invoke_method('SetAllowROMPortModeSolver', flag)

    def set_allow_discrete_port_solver(self, flag: bool = True):
        self.invoke_method('SetAllowDiscretePortSolver', flag)

    def set_use_rom_port_mode_solver_tet(
            self, general_purpose: bool = True, fast_reduced_order_model : bool = True):
        self.invoke_method('SetUseROMPortModeSolverTet', general_purpose, fast_reduced_order_model)

    def set_enable_native_single_ended(self, flag: bool = True):
        self.invoke_method('EnableNativeSingleEnded', flag)

    def set_use_deembedded_fields(self, flag: bool = True):
        self.invoke_method('UseDeembeddedFields', flag)

    def set_freq_dist_adapt_mode(self, mode: str):
        self.invoke_method('FreqDistAdaptMode', mode)

    def set_prefer_lean_output(self, flag: bool = True):
        self.invoke_method('SetPreferLeanOutput', flag)

    def set_use_imp_line_impedance_as_reference(self, flag: bool = True):
        self.invoke_method('SetUseImpLineImpedanceAsReference', flag)

    def set_use_orient_port_with_mask(self, flag: bool = True):
        self.invoke_method('SetUseOrientPortWithMask', flag)

    def set_consider_port_losses_tet(self, flag: bool = True):
        self.invoke_method('ConsiderPortLossesTet', flag)

    def set_stop_sweep_if_criterion_met(self, flag: bool = True):
        self.invoke_method('SetStopSweepIfCriterionMet', flag)

    def set_s_params_sweep_threshold(self, threshold: float):
        self.invoke_method('SetSweepThreshold', 'S-Parameters', threshold)

    def set_probes_sweep_threshold(self, threshold: float):
        self.invoke_method('SetSweepThreshold', 'Probes', threshold)

    def set_use_s_params_sweep_threshold(self, flag: bool = True):
        self.invoke_method('UseSweepThreshold', 'S-Parameters', flag)

    def set_use_probes_sweep_threshold(self, flag: bool = True):
        self.invoke_method('UseSweepThreshold', 'Probes', flag)

    def set_type(self, solver_type: str):
        self.invoke_method('Type', solver_type)

    def set_store_results_in_cache(self, flag: bool = True):
        self.invoke_method('StoreResultsInCache', flag)

    def set_method(self, mesh_method: str, sweep_method: str):
        self.invoke_method('SetMethod', mesh_method, sweep_method)

    def set_auto_norm_impedance(self, flag: bool = True):
        self.invoke_method('AutoNormImpedance', flag)

    def set_norming_impedance(self, impedance: float):
        self.invoke_method('NormingImpedance', impedance)

    def set_stimulation(self, port: Union[str, int], mode: Union[str, int]):
        self.invoke_method('Stimulation', port, mode)

    def set_number_of_sweep_error_checks(self, number: int):
        self.invoke_method('SweepErrorChecks', number)

    def set_minimum_number_of_sweep_samples(self, number: int):
        self.invoke_method('SweepMinimumSamples', number)

    def set_sweep_consider_all(self, flag: bool = True):
        self.invoke_method('SweepConsiderAll', flag)

    def reset_sweep_consider_list(self):
        self.invoke_method('SweepConsiderReset')

    def consider_sweep_s_param(self, port_out: int, mode_out: int, mode_in: int):
        self.invoke_method('SweepConsiderSPar', port_out, mode_out, mode_in)

    def set_minimum_number_of_result_data_samples(self, number: int):
        self.invoke_method('SetNumberOfResultDataSamples', number)

    def set_result_data_sampling_mode(self, mode: str):
        self.invoke_method('SetResultDataSamplingMode', mode)

    def set_extrude_open_boundary_condition(self, flag: bool = True):
        self.invoke_method('ExtrudeOpenBC', flag)

    def set_td_compatible_materials(self, flag: bool = True):
        self.invoke_method('TDCompatibleMaterials', flag)

    def set_calculate_static_b_field(self, flag: bool = True):
        self.invoke_method('CalcStatBField', flag)

    def set_calculate_power_loss(self, flag: bool = True):
        self.invoke_method('CalcPowerLoss', flag)

    def set_calculate_power_loss_per_component(self, flag: bool = True):
        self.invoke_method('CalcPowerLossPerComponent', flag)

    def set_use_green_sandy_ferrite_model(self, flag: bool = True):
        self.invoke_method('SetUseGreenSandyFerriteModel', flag)

    def set_green_sandy_threshold_h_field(self, threshold: float = True):
        self.invoke_method('SetGreenSandyThresholdH', threshold)

    def reset_excitation_list(self):
        self.invoke_method('ResetExcitationList')

    def add_item_to_excitation_list(self, port: Union[str, int], mode: str):
        self.invoke_method('AddToExcitationList', port, mode)

    def set_use_parallelization(self, flag: bool = True):
        self.invoke_method('UseParallelization', flag)

    def set_max_number_of_cpu_threads(self, number: int):
        self.invoke_method('MaxCPUs', number)

    def set_max_number_of_cpu_devices(self, number: int):
        self.invoke_method('MaximumNumberOfCPUDevices', number)

    def set_sweep_weight_evanescent(self, weight: float):
        self.invoke_method('SweepWeightEvanescent', weight)

    def set_low_frequency_stabilization(self, flag: bool = True):
        self.invoke_method('LowFrequencyStabilization', flag)

    def set_order_tet(self, order: str):
        self.invoke_method('OrderTet', order)

    def set_mixed_order_tet(self, flag: bool = True):
        self.invoke_method('MixedOrderTet', flag)

    def set_order_srf(self, order: str):
        self.invoke_method('OrderSrf', order)

    def set_mixed_order_srf(self, flag: bool = True):
        self.invoke_method('MixedOrderSrf', flag)

    def set_use_distributed_computing(self, flag: bool = True):
        self.invoke_method('UseDistributedComputing', flag)

    def set_network_computing_strategy(self, strategy: str):
        self.invoke_method('NetworkComputingStrategy', strategy)

    def set_network_computing_job_count(self, count: int):
        self.invoke_method('NetworkComputingJobCount', count)

    def set_use_sensitivity_analysis(self, flag: bool = True):
        self.invoke_method('UseSensitivityAnalysis', flag)

    def set_use_double_precision(self, flag: bool = True):
        self.invoke_method('UseDoublePrecision', flag)

    def set_preconditioner_accuracy_int_eq(self, tolerance: float):
        self.invoke_method('PreconditionerAccuracyIntEq', tolerance)

    def set_min_mlffm_box_size(self, tolerance: float):
        self.invoke_method('MinMLFMMBoxSize', tolerance)

    def set_mlffm_accuracy(self, accuracy: str):
        self.invoke_method('MLFMMAccuracy', accuracy)

    def set_use_cfie_for_cpec_int_eq(self, flag: bool = True):
        self.invoke_method('UseCFIEForCPECIntEq', flag)

    def set_use_fast_rcs_sweep_int_eq(self, flag: bool = True):
        self.invoke_method('UseFastRCSSweepIntEq', flag)

    def set_mrcs_sweep_properties(
            self,
            phi_start: float, phi_end: float, num_phi_steps: int,
            theta_start: float, theta_end: float, num_theta_steps: int,
            e_inc_theta: float, e_inc_phi: float):
        self.invoke_method(
            'SetMRCSSweepProperties', phi_start, phi_end, num_phi_steps,
            theta_start, theta_end, num_theta_steps, e_inc_theta, e_inc_phi)

    def get_mrcs_sweep_properties(self):
        phi_start = w32com.create_ref_double()
        phi_end = w32com.create_ref_double()
        num_phi_steps = w32com.create_ref_int()
        theta_start = w32com.create_ref_double()
        theta_end = w32com.create_ref_double()
        num_theta_steps = w32com.create_ref_int()
        e_inc_theta = w32com.create_ref_double()
        e_inc_phi = w32com.create_ref_double()
        activation = w32com.create_ref_bool()
        self.invoke_method(
            'GetRCSSweepProperties', phi_start, phi_end, num_phi_steps,
            theta_start, theta_end, num_theta_steps, e_inc_theta, e_inc_phi, activation)
        return (
            phi_start.value, phi_end.value, num_phi_steps.value,
            theta_start.value, theta_end.value, num_theta_steps.value,
            e_inc_theta.value, e_inc_phi.value, activation.value )

    def set_calc_block_excitation_in_parallel(
            self, enable: bool, use_block: bool, max_parallel: int):
        self.invoke_method('SetCalcBlockExcitationsInParallel', enable, use_block, max_parallel)

    def set_write_3d_fields_for_farfield_calc(self, flag: bool = True):
        self.invoke_method('SetWrite3DFieldsForFarfieldCalc', flag)

    def set_record_unit_cell_scan_farfield_on(self):
        self.invoke_method('SetRecordUnitCellScanFarfield', 'on')

    def set_record_unit_cell_scan_farfield_off(self):
        self.invoke_method('SetRecordUnitCellScanFarfield', 'off')

    def set_record_unit_cell_scan_farfield_auto(self):
        self.invoke_method('SetRecordUnitCellScanFarfield', 'auto')

    def set_record_unit_cell_scan_farfield(self, flag: bool = True):
        self.invoke_method('SetRecordUnitCellScanFarfield', 'on' if flag else 'off')

    def set_write_3d_fields_for_farfield_calc(self, flag_theta: bool = True, flag_phi: bool = True):
        self.invoke_method('SetConsiderUnitCellScanFarfieldSymmetry', flag_theta, flag_phi)

    def set_disable_result_templates_during_unit_cell_scan_angle_sweep(self, flag: bool = True):
        self.invoke_method('SetDisableResultTemplatesDuringUnitCellScanAngleSweep', flag)

    def extract_unit_cell_scan_farfield(
            self, frequencies: list[float], names: list[str], anchor: int):
        freq_str = ';'.join(str(s) for s in frequencies)
        names_str = ';'.join(str(s) for s in names)
        self.invoke_method('ExtractUnitCellScanFarfield', freq_str, names_str, anchor)

    def update_interpolation_settings(self, navigation_tree_path: str):
        self.invoke_method('UpdateInterpolationSettings', navigation_tree_path)

    def export_mor_solution(self, frequency: float):
        self.invoke_method('ExportMORSolution', frequency)

    def set_force_recalculate_old_samples(self, flag: bool = True):
        self.invoke_method('ForceRecalculateOldSamples', flag)

    def set_allow_float_direct_solver(self, flag: bool = True):
        self.invoke_method('SetAllowFloatDirectSolver', flag)

    def set_allow_change_settings_for_schematic(self, flag: bool = True):
        self.invoke_method('SetAllowChangeSettingsForSchematic', flag)


from . import Project
from . import ComObjectWrapper
from . import w32com

class Anchorpoint(ComObjectWrapper):
    def __init__(self, project: Project) -> None:
        self.project = project
        self.com_object = project.com_object.Anchorpoint

    def invoke_method(self, name, *args, **kwargs):
        self.project.ensure_active()
        return super().invoke_method(name, *args, **kwargs)

    def store(self, name: str):
        self.invoke_method('Store', name)

    def define(
            self, name: str,
            pos_x: float, pos_y: float, pos_z: float,
            normal_x: float, normal_y: float, normal_z: float,
            u_vector_x: float, u_vector_y: float, u_vector_z: float):
        self.invoke_method(
            'Define', name,
            pos_x, pos_y, pos_z,
            normal_x, normal_y, normal_z,
            u_vector_x, u_vector_y, u_vector_z)

    def restore(self, name: str):
        self.invoke_method('Restore', name)

    def delete(self, name: str):
        self.invoke_method('Delete', name)

    def rename(self, old_name: str, new_name: str):
        self.invoke_method('Rename', old_name, new_name)

    def create_folder(self, name: str):
        self.invoke_method('NewFolder', name)

    def delete_folder(self, name: str):
        self.invoke_method('DeleteFolder', name)

    def rename_folder(self, old_name: str, new_name: str):
        self.invoke_method('RenameFolder', old_name, new_name)

    def does_exist(self, name: str) -> bool:
        return self.invoke_method('DoesExist', name)

    def get_origin(self, name: str):
        x = w32com.create_ref_double()
        y = w32com.create_ref_double()
        z = w32com.create_ref_double()
        success = self.invoke_method('GetOrigin', name, x, y, z)
        return (x.value, y.value, z.value) if success else None

    def get_normal(self, name: str):
        x = w32com.create_ref_double()
        y = w32com.create_ref_double()
        z = w32com.create_ref_double()
        success = self.invoke_method('GetNormal', name, x, y, z)
        return (x.value, y.value, z.value) if success else None

    def get_u_vector(self, name: str):
        x = w32com.create_ref_double()
        y = w32com.create_ref_double()
        z = w32com.create_ref_double()
        success = self.invoke_method('GetUVector', name, x, y, z)
        return (x.value, y.value, z.value) if success else None


GEOMETRY_METER = 'm'
GEOMETRY_CENTIMETER = 'cm'
GEOMETRY_MILLIMETER = 'mm'
GEOMETRY_MICROMETER = 'um'
GEOMETRY_NANOMETER = 'nm'
GEOMETRY_FEET = 'ft'
GEOMETRY_INCH = 'in'
GEOMETRY_MIL = 'mil'

TIME_FEMTOSECOND = 'fs'
TIME_PICOSECOND = 'ps'
TIME_NANOSECOND = 'ns'
TIME_MICROSECOND = 'us'
TIME_MILLISECOND = 'ms'
TIME_SECOND = 's'

FREQUENCY_HERTZ = 'Hz'
FREQUENCY_KILOHERTZ = 'kHz'
FREQUENCY_MEGAHERTZ = 'MHz'
FREQUENCY_GIGAHERTZ = 'GHz'
FREQUENCY_TERAHERTZ = 'THz'
FREQUENCY_PETAHERTZ = 'PHz'

TEMPERATURE_CELSIUS = 'Celsius'
TEMPERATURE_KELVIN = 'Kelvin'
TEMPERATURE_FAHRENHEIT = 'Fahrenheit'


from . import Shape, IVBAProvider

class Cylinder(Shape):
    AXIS_X = 'x'
    AXIS_Y = 'y'
    AXIS_Z = 'z'

    def __init__(self, vbap: IVBAProvider) -> None:
        super().__init__(vbap, 'Cylinder')

    def set_axis(self, axis: str):
        self.record_method('Axis', axis)

    def set_outer_radius(self, radius: float):
        self.record_method('Outerradius', radius)

    def set_inner_radius(self, radius: float):
        self.record_method('Innerradius', radius)

    def set_x_center(self, center: float):
        self.record_method('Xcenter', center)

    def set_y_center(self, center: float):
        self.record_method('Ycenter', center)

    def set_z_center(self, center: float):
        self.record_method('Zcenter', center)

    def set_x_range(self, x_min: float, x_max: float):
        self.record_method('Xrange', x_min, x_max)

    def set_y_range(self, y_min: float, y_max: float):
        self.record_method('Yrange', y_min, y_max)

    def set_z_range(self, z_min: float, z_max: float):
        self.record_method('Zrange', z_min, z_max)

    def set_num_segments(self, num_segments: int):
        self.record_method('Segments', num_segments)

    def set_smooth_geometry(self):
        self.set_num_segments(0)



class ResultItem:
    def __init__(self, native_obj) -> None:
        self.native_obj = native_obj

    def get_data(self) -> list:
        return self.native_obj.get_data()

    def get_parameter_combination(self) -> dict:
        return self.native_obj.get_parameter_combination()

    def get_ref_imp_data(self) -> list:
        return self.native_obj.get_ref_imp_data()

    def get_x_data(self) -> list:
        return self.native_obj.get_xdata()

    def get_y_data(self) -> list:
        return self.native_obj.get_ydata()

    def get_length(self) -> int:
        return self.native_obj.length

    def get_run_id(self) -> int:
        return self.native_obj.run_id

    def get_title(self) -> str:
        return self.native_obj.title

    def get_tree_path(self) -> str:
        return self.native_obj.tree_path

    def get_x_label(self) -> str:
        return self.native_obj.xlabel

    def get_y_label(self) -> str:
        return self.native_obj.ylabel


from . import IVBAProvider, VBATypeName
from typing import Optional, Union

class VBAObjWrapper:
    def __init__(self, vbap: IVBAProvider, native_obj_name: str) -> None:
        self.history_list = vbap.get_history_list_generator()
        self.vbap = vbap
        self.native_obj_name = native_obj_name

    def record_method(self, name: str, *args: Union[str, int, float, bool]) -> None:
        self.history_list.invoke_method(self.native_obj_name, name, *args)

    def invoke_method(self, name: str, *args: Union[str, int, float, bool]) -> None:
        self.vbap.invoke_method(self.native_obj_name, name, *args)

    def query_method_str(self, name: str, *args: Union[str, int, float, bool]) -> str:
        return self.vbap.query_method(self.native_obj_name, name, *args)

    def query_method_int(self, name: str, *args: Union[str, int, float, bool]) -> int:
        return int(self.query_method_str(name, *args))

    def query_method_float(self, name: str, *args: Union[str, int, float, bool]) -> float:
        return float(self.query_method_str(name, *args))

    def query_method_bool(self, name: str, *args: Union[str, int, float, bool]) -> bool:
        return self.query_method_str(name, *args) != 'False'

    def query_method_t(
            self, name: str, return_type: Optional[VBATypeName],
            *args: Union[str, int, float, bool, VBATypeName]) -> tuple:
        return self.vbap.query_method_t(self.native_obj_name, name, return_type, *args)


from . import IVBAProvider, VBAObjWrapper

class Monitor(VBAObjWrapper):
    FIELD_TYPE_E_FIELD = 'Efield'
    FIELD_TYPE_H_FIELD = 'Hfield'
    FIELD_TYPE_POWER_FLOW = 'Powerflow'
    FIELD_TYPE_CURRENT = 'Current'
    FIELD_TYPE_POWER_LOSS = 'Powerloss'
    FIELD_TYPE_E_ENERGY = 'Eenergy'
    FIELD_TYPE_H_ENERGY = 'Henergy'
    FIELD_TYPE_FARFIELD = 'Farfield'
    FIELD_TYPE_FIELD_SOURCE = 'Fieldsource'
    FIELD_TYPE_SPACE_CHARGE = 'Spacecharge'
    FIELD_TYPE_PARTICLE_CURRENT_DENSITY = 'Particlecurrentdensity'

    DIMENSION_PLANE = 'plane'
    DIMENSION_VOLUME = 'volume'

    DIRECTION_X = 'x'
    DIRECTION_Y = 'y'
    DIRECTION_Z = 'z'

    DOMAIN_FREQUENCY = 'frequency'
    DOMAIN_TIME = 'time'
    DOMAIN_STATIC = 'static'

    ACCURACY_LOW = '1e-3'
    ACCURACY_MEDIUM = '1e-4'
    ACCURACY_HIGH = '1e-5'

    ORIGIN_BOUNDING_BOX = 'bbox'
    ORIGIN_ZERO = 'zero'
    ORIGIN_CUSTOM = 'free'

    SAMPLING_STRATEGY_FREQUENCIES = 'Frequencies'
    SAMPLING_STRATEGY_STEPS = 'Steps'
    SAMPLING_STRATEGY_LINEAR = 'Linear'
    SAMPLING_STRATEGY_LOGARITHMIC = 'Logarithmic'

    def __init__(self, vbap: IVBAProvider) -> None:
        super().__init__(vbap, 'Monitor')

    def reset(self):
        self.record_method('Reset')

    def set_name(self, name: str):
        self.record_method('Name', name)

    def rename(self, old_name: str, new_name: str):
        self.record_method('Rename', old_name, new_name)

    def delete(self, name: str):
        self.record_method('Delete', name)

    def create(self):
        self.record_method('Create')

    def set_field_type(self, field_type: str):
        self.record_method('FieldType', field_type)

    def set_dimension(self, dimension: str):
        self.record_method('Dimension', dimension)

    def set_plane_normal_direction(self, direction: str):
        self.record_method('PlaneNormal', direction)

    def set_plane_position(self, position: float):
        self.record_method('PlanePosition', position)

    def set_domain(self, domain: str):
        self.record_method('Domain', domain)

    def set_frequency(self, frequency: float):
        self.record_method('Frequency', frequency)

    def set_time_start(self, start: float):
        self.record_method('Tstart', start)

    def set_time_step(self, step: float):
        self.record_method('Tstep', step)

    def set_time_end(self, end: float):
        self.record_method('Tend', end)

    def set_use_time_end(self, flag: bool = True):
        self.record_method('UseTend', flag)

    def set_average_over_time(self, flag: bool = True):
        self.record_method('TimeAverage', flag)

    def set_repetition_period(self, period: float):
        self.record_method('RepetitionPeriod', period)

    def set_automatic_order(self, flag: bool = True):
        self.record_method('AutomaticOrder', flag)

    def set_max_order(self, order: int):
        self.record_method('MaxOrder', order)

    def set_number_of_frequency_samples(self, number: int):
        self.record_method('FrequencySamples', number)

    def set_compute_transient_farfield(self, flag: bool = True):
        self.record_method('TransientFarfield', flag)

    def set_accuracy(self, accuracy: str):
        self.record_method('Accuracy', accuracy)

    def set_origin(self, origin: str):
        self.record_method('Origin', origin)

    def set_custom_origin(self, x: float, y: float, z: float):
        self.record_method('UserOrigin', x, y, z)

    def set_frequency_range(self, f_min: float, f_max: float):
        self.record_method('FrequencyRange', f_min, f_max)

    def set_domain(self, domain: str):
        self.record_method('Domain', domain)

    def set_number_of_samples(self, number: int):
        self.record_method('Samples', number)

    def set_sampling_strategy(self, strategy: str):
        self.record_method('SamplingStrategy', strategy)

    def set_monitor_value(self, value: float):
        self.record_method('MonitorValue', value)

    def set_monitor_value_list(self, frequencies: list[float]):
        freq_str = ';'.join(str(s) for s in frequencies)
        print('freq_str=', freq_str)
        self.record_method('MonitorValueList', freq_str)

    def set_sampling_step(self, step: float):
        self.record_method('SampleStep', step)

    def set_monitor_value_range(self, f_min: float, f_max: float):
        self.record_method('MonitorValueRange', f_min, f_max)

    def set_use_subvolume(self, flag: bool = True):
        self.record_method('UseSubvolume', flag)

    def set_subvolume(
            self, x_min: float, x_max: float, y_min: float, y_max: float,
            z_min: float, z_max: float):
        self.record_method('SetSubvolume', x_min, x_max, y_min, y_max, z_min, z_max)

    def set_invert_orientation(self, flag: bool = True):
        self.record_method('InvertOrientation', flag)

    def set_export_farfield_source(self, flag: bool = True):
        self.record_method('ExportFarfieldSource', flag)

    def set_enable_near_calculation(self, flag: bool = True):
        self.record_method('EnableNearfieldCalculation', flag)

    def create_using_arbitrary_values(self, frequencies: list[float]):
        freq_str = ';'.join(str(s) for s in frequencies)
        self.record_method('CreateUsingArbitraryValues', freq_str)

    def create_using_linear_samples(self, f_min: float, f_max: float, num_samples: int):
        self.record_method('CreateUsingLinearSamples', f_min, f_max, num_samples)

    def create_using_linear_step(self, f_min: float, f_max: float, f_step: float):
        self.record_method('CreateUsingLinearStep', f_min, f_max, f_step)

    def create_using_log_samples(self, f_min: float, f_max: float, num_samples: int):
        self.record_method('CreateUsingLogSamples', f_min, f_max, num_samples)

    def export(self, excitation_name: str, file_path: str, flag: bool = True):
        #TODO: find out the meaning of the flag
        self.record_method('Export', 'nfs', excitation_name, file_path, flag)

    def set_subvolume_sampling(self, x: float, y: float, z: float):
        self.record_method('SetSubVolumeSampling', x, y, z)

    def change_subvolume_sampling(self, monitor_name: str, x: float, y: float, z: float):
        self.record_method('ChangeSubVolumeSampling', monitor_name, x, y, z)

    def change_subvolume_sampling_to_history(self, monitor_name: str, x: float, y: float, z: float):
        self.record_method('ChangeSubVolumeSamplingToHistory', monitor_name, x, y, z)

    def get_number_of_monitors(self) -> int:
        return self.query_method_int('GetNumberOfMonitors')

    def get_monitor_name_from_index(self, index: int) -> str:
        #TODO: check if index is one based or zero based
        return self.query_method_str('GetMonitorNameFromIndex', index)

    def get_monitor_type_from_index(self, index: int) -> str:
        #TODO: check if index is one based or zero based
        return self.query_method_str('GetMonitorTypeFromIndex', index)

    def get_monitor_domain_from_index(self, index: int) -> str:
        #TODO: check if index is one based or zero based
        return self.query_method_str('GetMonitorDomainFromIndex', index)

    def get_monitor_frequency_from_index(self, index: int) -> float:
        #TODO: check if index is one based or zero based
        return self.query_method_float('GetMonitorFrequencyFromIndex', index)

    def get_monitor_time_start_from_index(self, index: int) -> float:
        #TODO: check if index is one based or zero based
        return self.query_method_float('GetMonitorTstartFromIndex', index)

    def get_monitor_time_step_from_index(self, index: int) -> float:
        #TODO: check if index is one based or zero based
        return self.query_method_float('GetMonitorTstepFromIndex', index)

    def get_monitor_time_end_from_index(self, index: int) -> float:
        #TODO: check if index is one based or zero based
        return self.query_method_float('GetMonitorTendFromIndex', index)


from . import IVBAProvider, VBAObjWrapper

class SolverParameter(VBAObjWrapper):
    SOLVER_TYPE_TRANSIENT = 'Transient solver'
    SOLVER_TYPE_FREQ_DOMAIN = 'Frequency domain solver'
    SOLVER_TYPE_ASYMPTOTIC = 'Asymptotic solver'
    SOLVER_TYPE_EIGENMODE = 'Eigenmode solver'
    SOLVER_TYPE_ELECTROSTATICS = 'Electrostatics solver'
    SOLVER_TYPE_MAGNETOSTATIC = 'Magnetostatic solver'
    SOLVER_TYPE_LF_FREQ_DOMAIN = 'LF Frequency domain solver'
    SOLVER_TYPE_LF_FREQ_DOMAIN_EQS = 'LF Frequency domain solver (EQS)'
    SOLVER_TYPE_STATIONARY_CURRENT = 'Stationary current solver'
    SOLVER_TYPE_PARTICLE_TRACKING = 'Particle tracking solver'
    SOLVER_TYPE_PIC = 'PIC solver'
    SOLVER_TYPE_THERMAL = 'Thermal solver'
    SOLVER_TYPE_INTEGRAL_EQUATION = 'Integral equation solver'
    SOLVER_TYPE_MULTILAYER = 'Multilayer solver'
    SOLVER_TYPE_LF_TIME_DOMAIN_MQS = 'LF Time domain solver (MQS)'
    SOLVER_TYPE_LF_TIME_DOMAIN_EQS = 'LF Time domain solver (EQS)'
    SOLVER_TYPE_THERMAL_TRANSIENT = 'Thermal transient solver'
    SOLVER_TYPE_STRUCTURAL_MECHANICS = 'Structural mechanics solver'
    SOLVER_TYPE_WAKEFIELD = 'Wakefield solver'

    MESH_HEXAHEDRAL = 'Hexahedral'
    MESH_TETRAHEDRAL = 'Tetrahedral'
    MESH_SURFACE = 'Surface'

    def __init__(self, vbap: IVBAProvider) -> None:
        super().__init__(vbap, 'SolverParameter')

    def set_solver_type(self, solver_type: str, mesh: str):
        self.record_method('SolverType', solver_type, mesh)

    def set_ignore_lossy_metals(self, flag: bool = True):
        self.record_method('IgnoreLossyMetals', flag)

    def set_ignore_lossy_dielectrics(self, flag: bool = True):
        self.record_method('IgnoreLossyDielectrics', flag)

    def set_ignore_lossy_metals_for_wires(self, flag: bool = True):
        self.record_method('IgnoreLossyMetalsForWires', flag)

    def set_ignore_nonlinear_materials(self, flag: bool = True):
        self.record_method('IgnoreNonlinearMaterials', flag)

    def set_use_thin_wire_model(self, flag: bool = True):
        self.record_method('UseThinWireModel', flag)

    def set_use_zero_wire_radius(self, flag: bool = True):
        self.record_method('UseZeroWireRadius', flag)


from . import Project
from . import ComObjectWrapper
from . import w32com
from typing import Optional

class LumpedElement(ComObjectWrapper):
    TYPE_RLC_PARALLEL = 'rlcparallel'
    TYPE_RLC_SERIAL = 'rlcserial'
    TYPE_DIODE = 'diode'
    TYPE_SPICE_CIRCUIT = 'spicecircuit'
    TYPE_TOUCHSTONE = 'touchstone'
    TYPE_MULTIPIN_GROUP_ITEM = 'multipingroupitem'
    TYPE_MULTIPIN_GROUP_SPICE = 'multipingroupspice'
    TYPE_MULTIPIN_GROUP_TOUCHSTONE = 'multipingrouptouchstone'

    def __init__(self, project) -> None:
        self.project: Project = project
        self.com_object = project.com_object.LumpedElement

    def invoke_method(self, name, *args, **kwargs):
        self.project.ensure_active()
        return super().invoke_method(name, *args, **kwargs)

    def reset(self):
        self.invoke_method('Reset')

    def create(self):
        self.invoke_method('Create')

    def create_multipin(self):
        self.invoke_method('CreateMultipin')

    def modify(self):
        self.invoke_method('Modify')

    def set_type(self, elem_type: str):
        self.invoke_method('SetType', elem_type)

    def set_name(self, name: str):
        self.invoke_method('SetName', name)

    def set_folder_name(self, folder_name: str):
        self.invoke_method('Folder', folder_name)

    def set_resistance(self, value: float):
        self.invoke_method('SetR', value)

    def set_inductance(self, value: float):
        self.invoke_method('SetL', value)

    def set_capacitance(self, value: float):
        self.invoke_method('SetC', value)

    def set_blocking_conductivity(self, value: float):
        self.invoke_method('SetGs', value)

    def set_reverse_current(self, value: float):
        self.invoke_method('SetI0', value)

    def set_temperature_kelvin(self, value: float):
        self.invoke_method('SetT', value)

    def set_temperature_celsius(self, value: float):
        self.invoke_method('SetT', value-273.15)

    def set_temperature_fahrenheit(self, value: float):
        self.invoke_method('SetT', (value + 459.67)*(5/9))

    def set_point1(self, x: float, y: float, z: float):
        self.invoke_method('SetP1', False, x, y, z)

    def set_pick_as_point1(self):
        self.invoke_method('SetP1', True, 0, 0, 0)

    def set_point2(self, x: float, y: float, z: float):
        self.invoke_method('SetP2', False, x, y, z)

    def set_pick_as_point2(self):
        self.invoke_method('SetP2', True, 0, 0, 0)

    def get_coordinates(self, name: str):
        x0 = w32com.create_ref_double()
        y0 = w32com.create_ref_double()
        z0 = w32com.create_ref_double()
        x1 = w32com.create_ref_double()
        y1 = w32com.create_ref_double()
        z1 = w32com.create_ref_double()
        success = self.invoke_method('GetCoordinates', name, x0, y0, z0, x1, y1, z1)
        return (x0.value, y0.value, z0.value, x1.value, y1.value, z1.value) if success else None

    def get_properties(self, name: str):
        elem_type = w32com.create_ref_str()
        resistance = w32com.create_ref_double()
        inductance = w32com.create_ref_double()
        capacity = w32com.create_ref_double()
        blocking_conductivity = w32com.create_ref_double()
        reverse_current = w32com.create_ref_double()
        temperature = w32com.create_ref_double()
        radius = w32com.create_ref_double()
        success = self.invoke_method(
            'GetProperties', name, elem_type, resistance, inductance, capacity,
            blocking_conductivity, reverse_current, temperature, radius)
        return (
            elem_type.value, resistance.value, inductance.value, capacity.value,
            blocking_conductivity.value, reverse_current.value, temperature.value,
            radius.value) if success else None

    def set_circuit_file_name(self, file_name: str):
        self.invoke_method('CircuitFileName', file_name)

    def set_use_relative_path(self, flag: bool = True):
        self.invoke_method('UseRelativePath', flag)

    def set_circuit_id(self, id: int):
        self.invoke_method('CircuitId', id)

    def set_use_copy_only(self, flag: bool = True):
        self.invoke_method('UseCopyOnly', flag)

    def set_invert(self, flag: bool = True):
        self.invoke_method('SetInvert', flag)

    def set_monitor(self, flag: bool = True):
        self.invoke_method('SetMonitor', flag)

    def set_radius(self, radius: float):
        self.invoke_method('SetRadius', radius)

    def set_wire(self, wire: str):
        self.invoke_method('Wire', wire)

    def set_wire_end_to_end1(self):
        self.invoke_method('Position', 'end1')

    def set_wire_end_to_end2(self):
        self.invoke_method('Position', 'end2')

    def set_port_name(self, name: str):
        self.invoke_method('PortName', name)

    def start_name_iteration(self):
        self.invoke_method('StartLumpedElementNameIteration')

    def connect_multipin_element_pin_to_sub_element(
            self, multipin_name: str, circuit_pin_name: str, multipin_sub_element_name: str):
        self.invoke_method(
            'ConnectMultipinElementPinToSubElement', multipin_name, circuit_pin_name,
            multipin_sub_element_name)

    def connect_multipin_element_pin_to_short(self, multipin_name: str, circuit_pin_name: str):
        self.invoke_method('ConnectMultipinElementPinToShort', multipin_name, circuit_pin_name)

    def connect_multipin_element_pin_to_open(self, multipin_name: str, circuit_pin_name: str):
        self.invoke_method('ConnectMultipinElementPinToOpen', multipin_name, circuit_pin_name)

    def get_next_lumped_element_name(self) -> Optional[str]:
        name = w32com.create_ref_str()
        success = self.invoke_method('GetNextLumpedElementName', name)
        return name.value if success else None

    def rename(self, old_name: str, new_name: str):
        self.invoke_method('Rename', old_name, new_name)

    def delete(self, name: str):
        self.invoke_method('Delete', name)

    def create_folder(self, name: str):
        self.invoke_method('NewFolder', name)

    def delete_folder(self, name: str):
        self.invoke_method('DeleteFolder', name)

    def rename_folder(self, old_name: str, new_name: str):
        self.invoke_method('RenameFolder', old_name, new_name)


from . import Project
from . import Shape

class AnalyticalFace(Shape):
    def __init__(self, project: Project) -> None:
        super().__init__(project, project.com_object.AnalyticalFace)

    def set_law_x(self, expr: str):
        self.invoke_method('LawX', expr)

    def set_law_y(self, expr: str):
        self.invoke_method('LawY', expr)

    def set_law_z(self, expr: str):
        self.invoke_method('LawZ', expr)

    def set_param_range_u(self, u_min: float, u_max: float):
        self.invoke_method('ParameterRangeU', u_min, u_max)

    def set_param_range_v(self, v_min: float, v_max: float):
        self.invoke_method('ParameterRangeV', v_min, v_max)



from . import Interface
import os.path
from typing import Optional
from enum import Enum

MODULE_VERSION = '2.0'

class Project:
    class Type:
        from cst.interface import ProjectType
        DesignStudio = ProjectType.DS
        MicrowaveStudio = ProjectType.MWS
        EMStudio = ProjectType.EMS
        ParticleStudio = ProjectType.PS
        MphysicsStudio = ProjectType.MPS
        CableStudio = ProjectType.CS
        PCBStudio = ProjectType.PCBS
        FilterDesigner3D = ProjectType.FD3D

    def __init__(self, interface: Interface, native_obj):
        self.interface = interface
        self.native_obj = native_obj
        self.report_information(self.__get_welcome_message())

    def invoke_method(self, name, *args, **kwargs):
        self.ensure_active()
        return super().invoke_method(name, *args, **kwargs)

    def get_path(self):
        return self.path

    def set_use_history_list(self, flag: bool = True):
        self.use_history_list = flag

    def close_without_saving(self):
        # NOTE: officially undocumented
        self.interface.store_and_enable_quiet_mode()
        self.interface.com_object.CloseProject(self.get_path())
        self.interface.restore_quiet_mode()

    def close(self):
        # NOTE: officially undocumented
        self.invoke_method('Quit')

    def is_active(self):
        return self.path == self.__read_path()

    def activate(self):
        self.interface.com_object.OpenFile(self.path)

    def reset_all(self):
        self.invoke_method('ResetAll')

    def set_screen_updating(self, flag: bool = True):
        self.invoke_method('ScreenUpdating', flag)

    def select_quick_start_guide(self, guide_type: str):
        self.invoke_method('SelectQuickStartGuide', guide_type)

    def select_tree_item(self, item_name: str):
        self.interface.store_and_disable_quiet_mode()
        self.invoke_method('SelectTreeItem', item_name)
        self.interface.restore_quiet_mode()

    def get_number_of_selected_tree_items(self) -> int:
        return self.invoke_method('GetNumberOfSelectedTreeItems')

    def get_selected_tree_item(self) -> str:
        return self.invoke_method('GetSelectedTreeItem')

    def get_next_selected_tree_item(self) -> str:
        return self.invoke_method('GetNextSelectedTreeItem')

    def set_lock(self, flag: bool = True):
        self.invoke_method('SetLock', flag)

    def lock(self, flag: bool = True):
        self.invoke_method('SetLock', flag)

    def unlock(self):
        self.invoke_method('SetLock', False)

    def add_to_history(self, caption: str, contents: str):
        if not self.invoke_method('AddToHistory', caption, contents):
            raise Exception('Failed to add \''+caption+'\' to history!')

    def get_database_value(self, key) -> str:
        return self.invoke_method('GetDataBaseValue', key)

    def get_database_array_value(self, key, index) -> str:
        return self.invoke_method('GetDataBaseArrayValue', key, index)

    def set_solver_type(self, solver_type: str):
        self.invoke_method('ChangeSolverType', solver_type)

    def get_solver_type(self) -> str:
        return self.invoke_method('GetSolverType')

    def run_solver(self):
        if not self.invoke_method('RunSolver'):
            raise Exception('Failed to run solver!')

    def import_subproject(self, file_name: str, do_wcs_alignment: bool = False):
        error_msg = self.invoke_method('ImportSubProject', file_name, do_wcs_alignment)
        if isinstance(error_msg, str) and len(error_msg) > 0:
            raise Exception(error_msg)

    def backup(self, path: str):
        return self.invoke_method('Backup', path)

    def open_new_file(self):
        self.invoke_method('FileNew')
        self.__update_path()

    def open_file(self, path: str):
        self.invoke_method('OpenFile', path)
        self.__update_path()

    def save(self):
        self.invoke_method('Save')

    def save_as(self, new_path: str, include_results: bool = False):
        new_path = os.path.abspath(new_path)
        self.invoke_method('SaveAs', new_path, include_results)
        self.path = new_path

    def store_in_archive(
            self, path: str, keep_all_results: bool = True, keep_1d_results: bool = True,
            keep_farfield_data: bool = True, delete_project_folder: bool = False):
        self.invoke_method('StoreInArchive',
            path, keep_all_results, keep_1d_results, keep_farfield_data, delete_project_folder)

    def delete_parameter(self, name: str):
        self.invoke_method('DeleteParameter', name)

    def has_parameter(self, name: str) -> bool:
        return self.invoke_method('DoesParameterExist', name)

    def get_number_of_parameters(self) -> int:
        return self.invoke_method('GetNumberOfParameters')

    def get_parameter_name(self, index: int) -> str:
        return self.invoke_method('GetParameterName', index)

    def get_parameter_value(self, index: int) -> float:
        return self.invoke_method('GetParameterNValue', index)

    def get_parameter_expression(self, index: int) -> str:
        return self.invoke_method('GetParameterSValue', index)

    def rename_parameter(self, old_name: str, new_name: str):
        self.invoke_method('RenameParameter', old_name, new_name)

    def restore_parameter(self, name: str) -> str:
        return self.invoke_method('RestoreParameter', name)

    def restore_parameter_as_number(self, name: str) -> float:
        return self.invoke_method('RestoreDoubleParameter', name)

    def restore_parameter_expression(self, name: str) -> str:
        return self.invoke_method('RestoreParameterExpression', name)

    def store_parameter_with_description(self, name: str, expression: str, description: str):
        self.invoke_method('StoreParameterWithDescription', name, expression, description)

    def store_parameter(self, name: str, expression: str):
        self.invoke_method('StoreParameter', name, expression)

    def store_parameters(self, names: list[str], values: list[any]):
        names_arr = w32com.create_str_array(names)
        values_arr = w32com.create_str_array(values)
        self.invoke_method('StoreParameters', names_arr, values_arr)

    def store_numeric_parameter(self, name: str, value: float):
        self.invoke_method('StoreDoubleParameter', name, value)

    def ensure_parameter(self, name: str, default_expression: str):
        self.invoke_method('MakeSureParameterExists', name, default_expression)

    def does_project_depend_on_parameter(self, param_name: str) -> bool:
        return self.invoke_method('DoesProjectDependOnParameter', param_name)

    def set_parameter_description(self, name: str, description: str):
        self.invoke_method('SetParameterDescription', name, description)

    def get_parameter_description(self, name: str):
        return self.invoke_method('GetParameterDescription', name)

    def get_parameter_combination(self, result_id: str) -> Optional[tuple[any]]:
        names_var = w32com.create_ref_variant()
        values_var = w32com.create_ref_variant()
        success = self.invoke_method('GetParameterCombination', result_id, names_var, values_var)
        return (names_var.value, values_var.value) if success else None

    #TODO implement: def get_project_parameters(self, file_path: str)

    def clear_global_data_values(self):
        self.invoke_method('ClearGlobalDataValues')

    def delete_global_data_values(self, name: str):
        self.invoke_method('DeleteGlobalDataValue', name)

    def restore_global_data_value(self, name: str):
        return self.invoke_method('RestoreGlobalDataValue', name)

    def store_global_data_value(self, name: str, value):
        self.invoke_method('StoreGlobalDataValue', name, value)

    def reset_global_data_storage(self):
        self.invoke_method('ResetGlobalDataStorage')

    def set_global_data(self, name: str, value):
        self.invoke_method('SetGlobalData', name, value)

    def get_global_data(self, name: str):
        return self.invoke_method('GetGlobalData', name)

    # math functions won't be implemented, it's unnecessary

    def evaluate(self, expression):
        return self.invoke_method('Evaluate', expression)

    def set_activate_script_settings(self, switch: bool = True):
        self.invoke_method('ActivateScriptSettings', switch)

    def clear_script_settings(self):
        self.invoke_method('ClearScriptSettings')

    def get_last_0d_result(self, name: str) -> float:
        return self.invoke_method('GetLast0DResult', name)

    def get_last_1d_result(self, name: str):
        # TODO: return list[float]
        return self.invoke_method('GetLast1DResult', name)

    def get_last_1d_complex_result(self, name: str):
        # TODO: return list[complex]
        return self.invoke_method('GetLast1DComplexResult', name)

    def get_last_result_id(self) -> str:
        return self.invoke_method('GetLastResultID')

    def get_script_setting(self, name: str, default_value: str) -> str:
        return self.invoke_method('GetScriptSetting', name, default_value)

    def store_script_setting(self, name: str, value: str) -> str:
        self.invoke_method('StoreScriptSetting', name, value)

    def store_template_setting(self, name: str, value: str) -> str:
        self.invoke_method('StoreTemplateSetting', name, value)

    def get_script_file_name(self) -> str:
        return self.invoke_method('GetScriptFileName')

    def evaluate_result_templates(self):
        self.invoke_method('EvaluateResultTemplates')

    def set_application_name(self, name: str):
        self.invoke_method('SetApplicationName', name)

    def reset_application_name(self):
        self.invoke_method('ResetApplicationName')

    def reset_template_iterator(self):
        self.invoke_method('ResetTemplateIterator')

    def set_template_filter(self, filter_name: str, value: str):
        self.invoke_method('SetTemplateFilter', filter_name, value)

    def get_next_template(
            self, result_name: str, val_type: str, template_name: str, folder: str) -> bool:
        return self.invoke_method('GetNextTemplate', result_name, val_type, template_name, folder)

    def get_file_type(self, file_path: str) -> str:
        return self.invoke_method('GetFileType', file_path)

    def get_impedance_from_tree_item(self, tree_item_name: str):
        # TODO: return list[complex]
        return self.invoke_method('GetImpedanceFromTreeItem', tree_item_name)

    def get_first_table_result(self, result_name: str) -> str:
        return self.invoke_method('GetFirstTableResult', result_name)

    def get_next_table_result(self, result_name: str) -> str:
        return self.invoke_method('GetNextTableResult', result_name)

    def was_template_aborted(self) -> bool:
        return self.invoke_method('GetTemplateAborted')

    def get_macro_path(self) -> str:
        return self.invoke_method('GetMacroPath')

    def get_macro_path_from_index(self, index: int) -> str:
        return self.invoke_method('GetMacroPathFromIndex', index)

    def get_number_of_macro_paths(self) -> int:
        return self.invoke_method('GetNumberOfMacroPaths')

    def run_and_wait(self, command: str):
        self.invoke_method('RunAndWait', command)

    def run_macro(self, macro_name: str):
        self.invoke_method('RunMacro', macro_name)

    def run_script(self, script_name: str):
        self.invoke_method('RunScript', script_name)

    def report_information(self, *msg):
        self.invoke_method('ReportInformation', ''.join(list(map(lambda x: str(x), msg))))

    def report_warning(self, *msg):
        self.invoke_method('ReportWarning', ''.join(list(map(lambda x: str(x), msg))))

    def report_information_to_window(self, *msg):
        self.invoke_method('ReportInformationToWindow', ''.join(list(map(lambda x: str(x), msg))))

    def report_warning_to_window(self, *msg):
        self.invoke_method('ReportWarningToWindow', ''.join(list(map(lambda x: str(x), msg))))

    def report_error(self, *msg):
        self.invoke_method('ReportError', ''.join(list(map(lambda x: str(x), msg))))

    def set_common_mpi_cluster_config(
            self, install_folder: str, temp_folder: str, architecture: str):
        self.invoke_method('SetCommonMPIClusterConfig', install_folder, temp_folder, architecture)

    def set_common_mpi_cluster_login_info(self, user_name: str, private_key_file_path: str):
        self.invoke_method('SetCommonMPIClusterLoginInfo', user_name, private_key_file_path)

    def get_mpi_cluster_size(self) -> int:
        return self.invoke_method('GetMPIClusterSize')

    def clear_mpi_cluster_config(self):
        self.invoke_method('ClearMPIClusterConfig')

    def add_mpi_cluster_node_config(
            self, host_name: str, install_folder: str, temp_folder: str, architecture: str,
            active: bool = True):
        self.invoke_method(
            'AddMPIClusterNodeConfig', host_name, install_folder, temp_folder,
            architecture, active)

    def import_xy_curve_from_ascii_file(self, folder_name: str, file_path: str):
        self.invoke_method('ImportXYCurveFromASCIIFile', folder_name, file_path)

    def paste_curves_from_ascii_file(self, folder_name: str, file_path: str):
        self.invoke_method('PasteCurvesFromASCIIFile', folder_name, file_path)

    def paste_curves_from_clipboard(self, folder_name: str):
        self.invoke_method('PasteCurvesFromClipboard', folder_name)

    def scale_curves(self, folder_name: str, x_scale: float, y_scale: float):
        self.invoke_method('ScaleCurves', folder_name, x_scale, y_scale)

    def store_curves_in_ascii_files(self, file_name: str):
        self.invoke_method('StoreCurvesInASCIIFile', file_name)

    def store_curves_in_clipboard(self):
        self.invoke_method('StoreCurvesInClipboard')

    #TODO: implement CalculateFourierComplex
    #TODO: implement CalculateCONV
    #TODO: implement CalculateCROSSCOR
    #TODO: implement ApplyWindow

    def delete_results(self):
        self.invoke_method('DeleteResults')

    def get_field_frequency(self) -> float:
        return self.invoke_method('GetFieldFrequency')

    # TODO: implement missing methods

    def get_application_name(self) -> str:
        return self.invoke_method('GetApplicationName')

    def get_application_version(self) -> str:
        return self.invoke_method('GetApplicationVersion')

    def get_install_path(self) -> str:
        return self.invoke_method('GetInstallPath')

    def get_project_path(self, path_type: str) -> str:
        return self.invoke_method('GetProjectPath', path_type)

    # TODO: implement missing methods?

    def is_building_model(self) -> bool:
        return self.invoke_method('IsBuildingModel')

    def get_license_host_id(self) -> str:
        return self.invoke_method('GetLicenseHostId')

    def get_license_customer_number(self) -> str:
        return self.invoke_method('GetLicenseCustomerNumber')

    # TODO: implement missing methods

    def rebuild(self):
        self.invoke_method('Rebuild')

    def rebuild_on_parametric_change(self):
        show_msg_box = self.interface.is_in_quiet_mode()
        full_rebuild = False
        self.invoke_method('RebuildOnParametricChange', full_rebuild, show_msg_box)

    def rebuild_fully_on_parametric_change(self):
        show_msg_box = self.interface.is_in_quiet_mode()
        full_rebuild = True
        self.invoke_method('RebuildOnParametricChange', full_rebuild, show_msg_box)

    def get_pickpoint_coords(self, pickpoint_id: int):
        x = self.invoke_method('xp', pickpoint_id)
        y = self.invoke_method('yp', pickpoint_id)
        z = self.invoke_method('zp', pickpoint_id)
        return [x, y, z]

    # TODO: implement missing methods

    def export_image_to_file(self, path: str, width: int = 0, height: int = 0):
        self.invoke_method('ExportImageToFile', path, width, height)

    def export_image_to_clipboard(self, width: int = 0, height: int = 0):
        self.invoke_method('ExportImageToClipboard', width, height)

    # TODO: implement missing methods

    def ensure_active(self):
        if not self.is_active():
            self.activate()

    def __read_path(self):
        return super().invoke_method('GetProjectPath', 'Project') + '.cst'

    def __update_path(self):
        self.path = self.__read_path()

    def __get_welcome_message(self):
        return f'CST Studio Suite Python interface v{MODULE_VERSION} by Samuel Travnicek connected.'


from . import Project
from . import ComObjectWrapper
from . import w32com

class Curve(ComObjectWrapper):
    ITERATION_ALL = 'all'
    ITERATION_OPEN = 'open'
    ITERATION_CLOSED = 'closed'

    def __init__(self, project: Project) -> None:
        self.project = project
        self.com_object = project.com_object.Curve

    def invoke_method(self, name, *args, **kwargs):
        self.project.ensure_active()
        return super().invoke_method(name, *args, **kwargs)

    def create_curve(self, name: str):
        self.invoke_method('NewCurve', name)

    def delete_curve(self, name: str):
        self.invoke_method('DeleteCurve', name)

    def rename_curve(self, old_name: str, new_name: str):
        self.invoke_method('RenameCurve', old_name, new_name)

    def delete_curve_item(self, curve_name: str, item_name: str):
        self.invoke_method('DeleteCurveItem', curve_name, item_name)

    def rename_curve_item(self, curve_name: str, old_item_name: str, new_item_name: str):
        self.invoke_method('RenameCurveItem', curve_name, old_item_name, new_item_name)

    def delete_curve_item_segment(self, curve_name: str, edge_id: int):
        self.invoke_method('DeleteCurveItemSegment', curve_name, edge_id)

    def move_curve_item(self, item_name: str, old_curve_name: str, new_curve_name: str):
        self.invoke_method('MoveCurveItem', item_name, old_curve_name, new_curve_name)

    def start_curve_name_iteration(self, it_type: str):
        self.invoke_method('StartCurveNameIteration', it_type)

    def get_next_curve_name(self) -> str:
        return self.invoke_method('GetNextCurveName')

    def get_point_coordinates(self, item_name: str, pid: str):
        x = w32com.create_ref_double()
        y = w32com.create_ref_double()
        z = w32com.create_ref_double()
        success = self.invoke_method('GetPointCoordinates', item_name, pid, x, y, z)
        return (x.value, y.value, z.value) if success else None

    def get_number_of_points(self) -> int:
        return self.invoke_method('GetNumberOfPoints')

    def is_closed(self, item_name: str) -> bool:
        return self.invoke_method('IsClosed', item_name)



from . import Project
from . import ComObjectWrapper
from . import w32com
import numpy as np

class WCS(ComObjectWrapper):
    def __init__(self, project: Project) -> None:
        self.project = project
        self.com_object = project.com_object.WCS

    def invoke_method(self, name, *args, **kwargs):
        self.project.ensure_active()
        return super().invoke_method(name, *args, **kwargs)

    def activate_local(self):
        self.invoke_method('ActivateWCS', 'local')

    def activate_global(self):
        self.invoke_method('ActivateWCS', 'global')

    def store(self, name: str):
        self.invoke_method('Store', name)

    def restore(self, name: str):
        self.invoke_method('Restore', name)

    def delete(self, name: str):
        self.invoke_method('Delete', name)

    def rename(self, old_name: str, new_name: str):
        self.invoke_method('Rename', old_name, new_name)

    def set_normal(self, x: float, y: float, z: float):
        self.invoke_method('SetNormal', x, y, z)

    def set_origin(self, x: float, y: float, z: float):
        self.invoke_method('SetOrigin', x, y, z)

    def set_u_vector(self, x: float, y: float, z: float):
        self.invoke_method('SetUVector', x, y, z)

    def align_wcs_with_selected_point(self):
        self.invoke_method('AlignWCSWithSelected', 'Point')

    def align_wcs_with_selected_3_points(self):
        self.invoke_method('AlignWCSWithSelected', '3Points')

    def align_wcs_with_selected_edge(self):
        self.invoke_method('AlignWCSWithSelected', 'Edge')

    def align_wcs_with_selected_edge_center(self):
        self.invoke_method('AlignWCSWithSelected', 'EdgeCenter')

    def align_wcs_with_selected_rotation_edge(self):
        self.invoke_method('AlignWCSWithSelected', 'RotationEdge')

    def align_wcs_with_selected_face(self):
        self.invoke_method('AlignWCSWithSelected', 'Face')

    def rotate_wcs_around_u_deg(self, angle_deg: float):
        self.invoke_method('RotateWCS', 'u', angle_deg)

    def rotate_wcs_around_u_rad(self, angle_rad: float):
        self.rotate_wcs_around_u_deg(np.rad2deg(angle_rad))

    def rotate_wcs_around_v_deg(self, angle_deg: float):
        self.invoke_method('RotateWCS', 'v', angle_deg)

    def rotate_wcs_around_v_rad(self, angle_rad: float):
        self.rotate_wcs_around_v_deg(np.rad2deg(angle_rad))

    def rotate_wcs_around_w_deg(self, angle_deg: float):
        self.invoke_method('RotateWCS', 'w', angle_deg)

    def rotate_wcs_around_w_rad(self, angle_rad: float):
        self.rotate_wcs_around_w_deg(np.rad2deg(angle_rad))

    def move_local_wcs(self, du: float, dv: float, dw: float):
        self.invoke_method('MoveWCS', 'local', du, dv, dw)

    def move_global_wcs(self, du: float, dv: float, dw: float):
        self.invoke_method('MoveWCS', 'global', du, dv, dw)

    def align_wcs_with_global_coordinates(self):
        self.invoke_method('AlignWCSWithGlobalCoordinates')

    def set_workplane_size(self, size: float):
        self.invoke_method('SetWorkplaneSize', size)

    def set_workplane_raster(self, raster_size: float):
        self.invoke_method('SetWorkplaneRaster', raster_size)

    def set_workplane_snap(self, flag: bool = True):
        self.invoke_method('SetWorkplaneSnap', flag)

    def set_workplane_autoadjust(self, flag: bool = True):
        self.invoke_method('SetWorkplaneAutoadjust', flag)

    def set_workplane_snap_autoadjust(self, flag: bool = True):
        self.invoke_method('SetWorkplaneSnapAutoadjust', flag)

    def set_workplane_autosnap_factor(self, factor: float):
        self.invoke_method('SetWorkplaneAutosnapFactor', factor)

    def set_workplane_snap_raster(self, raster_size: float):
        self.invoke_method('SetWorkplaneSnapRaster', raster_size)

    def is_local_wcs_active(self) -> bool:
        return self.invoke_method('IsWCSActive') == 'local'

    def is_global_wcs_active(self) -> bool:
        return self.invoke_method('IsWCSActive') == 'global'

    def does_exist(self, wcs_name: str):
        return self.invoke_method('DoesExist', wcs_name)

    def get_origin(self, wcs_name: str):
        x = w32com.create_ref_double()
        y = w32com.create_ref_double()
        z = w32com.create_ref_double()
        self.invoke_method('GetOrigin', wcs_name, x, y, z)
        return (x.value, y.value, z.value)

    def get_normal(self, wcs_name: str):
        x = w32com.create_ref_double()
        y = w32com.create_ref_double()
        z = w32com.create_ref_double()
        self.invoke_method('GetNormal', wcs_name, x, y, z)
        return (x.value, y.value, z.value)

    def get_u_vector(self, wcs_name: str):
        x = w32com.create_ref_double()
        y = w32com.create_ref_double()
        z = w32com.create_ref_double()
        self.invoke_method('GetUVector', wcs_name, x, y, z)
        return (x.value, y.value, z.value)

    def get_affine_matrix_uvw_to_xyz(self, wcs_name: str):
        ux = w32com.create_ref_double()
        uy = w32com.create_ref_double()
        uz = w32com.create_ref_double()
        vx = w32com.create_ref_double()
        vy = w32com.create_ref_double()
        vz = w32com.create_ref_double()
        wx = w32com.create_ref_double()
        wy = w32com.create_ref_double()
        wz = w32com.create_ref_double()
        success = self.invoke_method(
            'GetAffineMatrixUVW2XYZ', wcs_name, ux, uy, uz, vx, vy, vz, wx, wy, wz)
        return (ux, uy, uz, vx, vy, vz, wx, wy, wz) if success else None

    def get_affine_matrix_xyz_to_uvw(self, wcs_name: str):
        ux = w32com.create_ref_double()
        uy = w32com.create_ref_double()
        uz = w32com.create_ref_double()
        vx = w32com.create_ref_double()
        vy = w32com.create_ref_double()
        vz = w32com.create_ref_double()
        wx = w32com.create_ref_double()
        wy = w32com.create_ref_double()
        wz = w32com.create_ref_double()
        success = self.invoke_method(
            'GetAffineMatrixXYZ2UVW', wcs_name, ux, uy, uz, vx, vy, vz, wx, wy, wz)
        return (ux, uy, uz, vx, vy, vz, wx, wy, wz) if success else None

    def get_wcs_point_from_global(self, wcs_name: str, x: float, y: float, z: float):
        u = w32com.create_ref_double()
        v = w32com.create_ref_double()
        w = w32com.create_ref_double()
        success = self.invoke_method('GetWCSPointFromGlobal', wcs_name, u, v, w, x, y, z)
        return (u.value, v.value, w.value) if success else None

    def get_global_point_from_wcs(self, wcs_name: str, u: float, v: float, w: float):
        x = w32com.create_ref_double()
        y = w32com.create_ref_double()
        z = w32com.create_ref_double()
        success = self.invoke_method('GetWCSPointFromGlobal', wcs_name, x, y, z, u, v, w)
        return (x.value, y.value, z.value) if success else None


from .CST import Project
import os.path
from typing import Optional
import os.path

def get_win_library_path(cst_version: int) -> str:
    return f'C:\\Program Files (x86)\\CST Studio Suite {cst_version}\\Library\\Materials'

def get_default_win_material_path(cst_version: int, mat_name: str) -> str:
    return os.path.join(get_win_library_path(cst_version), f'{mat_name}.mtd')

def get_library_path(project: Project) -> str:
    return os.path.join(project.get_install_path(), 'Library', 'Materials')

def get_material_path(project: Project, mat_name: str) -> str:
    return os.path.join(get_library_path(project), f'{mat_name}.mtd')

class Material:
    def __init__(self) -> None:
        self.commands = []
        self.curr_section = ''
        self.is_def_section = False
        self.custom_name = None
        self.loaded_name = ''

    def get_loaded_name(self) -> str:
        return self.loaded_name

    def set_custom_name(self, name: str):
        self.custom_name = name

    def get_custom_name(self) -> Optional[str]:
        return self.custom_name

    def load_from_file(self, file_path: str):
        file = open(file_path, 'r')
        self.__set_loaded_name(file_path)
        lines = file.readlines()
        for line in lines:
            self.__process_line(line)

    def __process_line(self, line: str):
        line = line.strip()
        if line.startswith('['):
            self.__process_section_line(line)
        elif len(line) == 0:
            pass
        elif self.is_def_section:
            self.__process_definition_line(line)
        else:
            pass

    def import_to_project(self, project: Project):
        project.get_modeler().add_to_history(
            f'Define material: {self.__get_name()}', self.__get_import_command())

    def __process_section_line(self, line: str):
        self.curr_section = line[1:-1]
        self.is_def_section = self.curr_section == 'Definition'

    def __process_definition_line(self, line: str):
        self.commands.append(line)

    def __get_import_command(self) -> str:
        return \
            'With Material\n'+\
            '.Reset\n'+\
            f'.Name "{self.__get_name()}"\n'+\
            '\n'.join(self.commands)+\
            '\nEnd With'

    def __set_loaded_name(self, file_path: str):
        self.loaded_name = os.path.basename(file_path).removesuffix('.mtd')

    def __get_name(self) -> str:
        if self.custom_name is not None:
            return self.custom_name
        else:
            return self.loaded_name


from . import IVBAProvider, VBAObjWrapper

class Loft(VBAObjWrapper):
    def __init__(self, vbap: IVBAProvider) -> None:
        super().__init__(vbap, 'Loft')

    def reset(self):
        self.record_method('Reset')

    def set_name(self, name: str):
        self.record_method('Name', name)

    def set_component(self, component_name: str):
        self.record_method('Component', component_name)

    def set_material(self, mat_name: str):
        self.record_method('Material', mat_name)

    def set_tangency(self, tang: float):
        self.record_method('Tangency', tang)

    def create(self):
        #TODO: check if Create or CreateNew
        self.record_method('Create')



from . import ComObjectWrapper
import numpy as np

class Result1DComplex(ComObjectWrapper):
    def __init__(self, com_object) -> None:
        self.com_object = com_object

    def get_num_elements(self) -> int:
        return self.invoke_method('GetN')

    def get_x_at(self, index: int) -> float:
        return self.invoke_method('GetX', index)

    def get_y_at(self, index: int) -> complex:
        y_re = self.invoke_method('GetYRe', index)
        y_im = self.invoke_method('GetYIm', index)
        return y_re + 1j*y_im

    def get_array(self, component: str):
        return np.array(self.invoke_method('GetArray', component))

    def get_x_values(self):
        return self.get_array('x')

    def get_y_values(self):
        re_vals = self.get_array('yre')
        im_vals = self.get_array('yim')
        return re_vals + 1j*im_vals


from . import IVBAProvider, VBAObjWrapper, VBATypeName
from typing import Optional

class DiscretePort(VBAObjWrapper):
    TYPE_S_PARAMETER = 'Sparameter'
    TYPE_VOLTAGE = 'Voltage'
    TYPE_CURRENT = 'Current'

    def __init__(self, vbap: IVBAProvider) -> None:
        super().__init__(vbap, 'DiscretePort')

    def reset(self):
        self.record_method('Reset')

    def create(self):
        self.record_method('Create')

    def modify(self):
        self.record_method('Modify')

    def set_number(self, number: int):
        self.record_method('PortNumber', number)

    def set_label(self, label: str):
        self.record_method('Label', label)

    def set_type(self, port_type: str):
        self.record_method('Type', port_type)

    def set_impedance(self, impedance: float):
        self.record_method('Impedance', impedance)

    def set_voltage(self, voltage: float):
        self.record_method('Voltage', voltage)

    def set_voltage_port_impedance(self, impedance: float):
        self.record_method('VoltagePortImpedance', impedance)

    def set_current(self, current: float):
        self.record_method('Current', current)

    def set_current_port_admittance(self, admittance: float):
        self.record_method('CurrentPortAdmittance', admittance)

    def set_radius(self, radius: float):
        self.record_method('Radius', radius)

    def set_point1(self, x: float, y: float, z: float):
        self.record_method('SetP1', False, x, y, z)

    def set_pick_as_point1(self):
        self.record_method('SetP1', True, 0, 0, 0)

    def set_point2(self, x: float, y: float, z: float):
        self.record_method('SetP2', False, x, y, z)

    def set_pick_as_point2(self):
        self.record_method('SetP2', True, 0, 0, 0)

    def set_invert_direction(self, flag: bool = True):
        self.record_method('InvertDirection', flag)

    def set_local_coordinates(self, flag: bool = True):
        self.record_method('LocalCoordinates', flag)

    def set_monitor(self, flag: bool = True):
        self.record_method('Monitor', flag)

    def set_wire(self, wire_name: str):
        self.record_method('Wire', wire_name)

    def set_wire_end_to_end1(self):
        self.record_method('Position', 'end1')

    def set_wire_end_to_end2(self):
        self.record_method('Position', 'end2')

    def set_lumped_element_name(self, name: str):
        self.record_method('LumpedElementName', name)

    def set_delete_lumped_element(self, flag: bool = True):
        self.record_method('DeleteLumpedElement', flag)

    # returns: (dir, index)
    def get_element_dir_index(self, port_number: int) -> tuple[int, int]:
        L = VBATypeName.Long
        return self.query_method_t('GetElementDirIndex', None, port_number, L, L)

    def get_element_second_index(self, port_number: int) -> int:
        L = VBATypeName.Long
        return self.query_method_t('GetElement2ndIndex', None, port_number, L)[0]

    def get_length(self, port_number: int) -> float:
        return self.query_method_float('GetLength', port_number)

    def get_grid_length(self, port_number: int) -> float:
        return self.query_method_float('GetGridLength', port_number)

    # returns: (x0, y0, z0, x1, y1, z1) or None
    def get_coordinates(self, port_number: int) \
            -> Optional[tuple[float, float, float, float, float, float]]:
        B, D = VBATypeName.Boolean, VBATypeName.Double
        res = self.query_method_t('GetCoordinates', B, port_number, D, D, D, D, D, D)
        return None if not res[0] else res[1:]

    # returns: (type, impedance, current, voltage, voltage_port_impedance, radius, monitor)
    def get_properties(self, port_number: int) \
            -> Optional[tuple[str, float, float, float, float, float, bool]]:
        S, D, B = VBATypeName.String, VBATypeName.Double, VBATypeName.Boolean
        res = self.query_method_t('GetProperties', B, port_number, S, D, D, D, D, D, B)
        return None if not res[0] else res[1:]

    def delete(self, port_number: int):
        self.record_method('Delete', port_number)


from . import Project
from . import ComObjectWrapper

class LumpedFaceElement(ComObjectWrapper):
    TYPE_RLC_PARALLEL = 'rlcparallel'
    TYPE_RLC_SERIAL = 'rlcserial'
    TYPE_DIODE = 'diode'
    TYPE_SPICE_CIRCUIT = 'spicecircuit'
    TYPE_TOUCHSTONE = 'touchstone'
    TYPE_MULTIPIN_GROUP_ITEM = 'multipingroupitem'
    TYPE_MULTIPIN_GROUP_SPICE = 'multipingroupspice'
    TYPE_MULTIPIN_GROUP_TOUCHSTONE = 'multipingrouptouchstone'

    def __init__(self, project: Project) -> None:
        self.project = project
        self.com_object = project.com_object.LumpedFaceElement

    def invoke_method(self, name, *args, **kwargs):
        self.project.ensure_active()
        return super().invoke_method(name, *args, **kwargs)

    def reset(self):
        self.invoke_method('Reset')

    def create(self):
        self.invoke_method('Create')

    def modify(self):
        self.invoke_method('Modify')

    def set_type(self, elem_type: str):
        self.invoke_method('Type', elem_type)

    def set_name(self, name: str):
        self.invoke_method('Name', name)

    def set_folder_name(self, folder_name: str):
        self.invoke_method('Folder', folder_name)

    def set_resistance(self, value: float):
        self.invoke_method('SetR', value)

    def set_inductance(self, value: float):
        self.invoke_method('SetL', value)

    def set_capacity(self, value: float):
        self.invoke_method('SetC', value)

    def set_blocking_conductivity(self, value: float):
        self.invoke_method('SetGs', value)

    def set_reverse_current(self, value: float):
        self.invoke_method('SetI0', value)

    def set_temperature_kelvin(self, value: float):
        self.invoke_method('SetT', value)

    def set_temperature_celsius(self, value: float):
        self.invoke_method('SetT', value-273.15)

    def set_temperature_fahrenheit(self, value: float):
        self.invoke_method('SetT', (value + 459.67)*(5/9))

    def set_point1(self, x: float, y: float, z: float):
        self.invoke_method('SetP1', False, x, y, z)

    def set_pick_as_point1(self):
        self.invoke_method('SetP1', True, 0, 0, 0)

    def set_point2(self, x: float, y: float, z: float):
        self.invoke_method('SetP2', False, x, y, z)

    def set_pick_as_point2(self):
        self.invoke_method('SetP2', True, 0, 0, 0)

    def set_circuit_file_name(self, file_name: str):
        self.invoke_method('CircuitFileName', file_name)

    def set_use_relative_path(self, flag: bool = True):
        self.invoke_method('UseRelativePath', flag)

    def set_circuit_id(self, id: int):
        self.invoke_method('CircuitId', id)

    def set_use_copy_only(self, flag: bool = True):
        self.invoke_method('UseCopyOnly', flag)

    def set_invert(self, flag: bool = True):
        self.invoke_method('SetInvert', flag)

    def set_monitor(self, flag: bool = True):
        self.invoke_method('SetMonitor', flag)

    def set_use_projection(self, flag: bool = True):
        self.invoke_method('UseProjection', flag)

    def set_reverse_projection(self, flag: bool = True):
        self.invoke_method('ReverseProjection', flag)

    def set_port_name(self, port_name: str):
        self.invoke_method('PortName', port_name)

    def set_circuit_file_name(self, file_name: str):
        self.invoke_method('CircuitFileName', file_name)

    def set_use_relative_path(self, flag: bool = True):
        self.invoke_method('UseRelativePath', flag)

    def set_circuit_id(self, id: int):
        self.invoke_method('CircuitId', id)

    def set_use_copy_only(self, flag: bool = True):
        self.invoke_method('UseCopyOnly', flag)

    def set_invert(self, flag: bool = True):
        self.invoke_method('SetInvert', flag)

    def set_monitor(self, flag: bool = True):
        self.invoke_method('SetMonitor', flag)

    def set_use_projection(self, flag: bool = True):
        self.invoke_method('UseProjection', flag)

    def set_reverse_projection(self, flag: bool = True):
        self.invoke_method('ReverseProjection', flag)

    def set_port_name(self, name: str):
        self.invoke_method('PortName', name)

    def set_delete_port(self, flag: bool = True):
        self.invoke_method('DeletePort', flag)

    def rename(self, old_name: str, new_name: str):
        self.invoke_method('Rename', old_name, new_name)

    def delete(self, name: str):
        self.invoke_method('Delete', name)

    def create_folder(self, name: str):
        self.invoke_method('NewFolder', name)

    def delete_folder(self, name: str):
        self.invoke_method('DeleteFolder', name)

    def rename_folder(self, old_name: str, new_name: str):
        self.invoke_method('RenameFolder', old_name, new_name)


from . import FarfieldExporter
import numpy as np

class UVSphereFarfieldExporter(FarfieldExporter):
    DEFAULT_STEP_DEG: float = 5.0

    def __init__(self) -> None:
        super().__init__()
        self.set_polar_angle_step_deg(UVSphereFarfieldExporter.DEFAULT_STEP_DEG) # theta
        self.set_lateral_angle_step_deg(UVSphereFarfieldExporter.DEFAULT_STEP_DEG) # phi
        self.radius = 1.0

    def set_polar_angles_deg(self, angles_deg: np.array):
        self.polar_angles = angles_deg

    def set_polar_angles_rad(self, angles_rad: np.array):
        self.set_polar_angles_deg(np.rad2deg(angles_rad))

    def set_lateral_angles_deg(self, angles_deg: np.array):
        self.lateral_angles = angles_deg

    def set_lateral_angles_rad(self, angles_rad: np.array):
        self.set_lateral_angles_deg(np.rad2deg(angles_rad))

    def set_polar_angle_step_deg(self, step_deg: float):
        self.set_polar_angles_deg(np.arange(0, 180+step_deg, step_deg))

    def set_polar_angle_step_rad(self, step_rad: float):
        self.set_polar_angle_step_deg(np.rad2deg(step_rad))

    def set_lateral_angle_step_deg(self, step_deg: float):
        self.set_lateral_angles_deg(np.arange(0, 360+step_deg, step_deg))

    def set_lateral_angle_step_rad(self, step_rad: float):
        self.set_lateral_angle_step_deg(np.rad2deg(step_rad))

    def set_radius(self, radius: float):
        self.radius = radius

    def export_abs(self):
        values = super().export_abs()
        return values.reshape((len(self.polar_angles), len(self.lateral_angles)))

    def export_complex_theta(self):
        values = super().export_complex_theta()
        return values.reshape((len(self.polar_angles), len(self.lateral_angles)))

    def export_complex_phi(self):
        values = super().export_complex_phi()
        return values.reshape((len(self.polar_angles), len(self.lateral_angles)))

    def __generate_points(self):
        self.clear_points()
        for polar_angle in self.polar_angles:
            for lateral_angle in self.lateral_angles:
                self.add_point(polar_angle, lateral_angle, self.radius)